/*  Gtk+ Widget Factory
 *  Copyright (C) 1999-2000  Jeroen Benckhuijsen
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include "gwf_projtree.h"
#include <gtk/gtk.h>
#include <gwf_lib.h>

typedef struct	_GwfProjTreeData	GwfProjTreeData;

struct _GwfProjTreeData
{
  GtkCTree	*tree;
  GwfProject	*project;
};

static void		gwf_projtree_class_init		(GwfProjTreeClass	*klass);
static void		gwf_projtree_init		(GwfProjTree		*projtree);
static void		gwf_projtree_destroy		(GtkObject		*object);

static void		gwf_projtree_construct_tree	(GwfProject *project,gpointer user_data);
static void		gwf_projtree_destroy_tree 	(GwfProject *project,gpointer user_data);

/************************/
/* Treedata interaction */
/************************/
static void		gwf_projtree_add_treedata		(GwfProjTree *projtree,GwfProject *project);
static void             gwf_projtree_create_foreach_treedata	(GwfTreeData *data,gpointer user_data);
static void		gwf_projtree_destroy_foreach_treedata	(GwfTreeData *data,gpointer user_data);

/*************************************/
/* Project and projectlist callbacks */
/*************************************/
static void	curproj_changed_cb	(GwfProjlist *projlist, gchar *name,gpointer user_data);
static void	project_added_cb	(GwfProjlist *projlist, gchar *name,gpointer user_data);
static void	project_removed_cb	(GwfProjlist *projlist, gchar *name,gpointer user_data);
static void	widname_changed_cb	(GwfProject *project, gchar *name,gpointer user_data);

/****************/
/* GUI Creation */
/****************/
static void		tree_row_selected		(GtkCTree *ctree,GList *node,gint column,gpointer user_data);
static void		gwf_projtree_interface 		(GwfProjTree *projtree);

guint
gwf_projtree_get_type (void)
{
  static guint gwf_projtree_type = 0;

  if (!gwf_projtree_type)
    {
      GtkTypeInfo gwf_projtree_info =
      {
	"GwfProjTree",
	sizeof (GwfProjTree),
	sizeof (GwfProjTreeClass),
	(GtkClassInitFunc) gwf_projtree_class_init,
	(GtkObjectInitFunc) gwf_projtree_init,
	/* reserved_1 */ NULL,
	/* reserved_2 */ NULL,
	(GtkClassInitFunc) NULL,
      };

      gwf_projtree_type = gtk_type_unique (gtk_scrolled_window_get_type(), &gwf_projtree_info);
    }

  return gwf_projtree_type;
}

static void		
gwf_projtree_class_init (GwfProjTreeClass *klass)
{
  ((GtkObjectClass *)klass)->destroy = gwf_projtree_destroy;
}

static void
gwf_projtree_init (GwfProjTree *projtree)
{
  gwf_projtree_interface(projtree);
  projtree->projects = gtk_ctree_add_node(projtree->projtree,NULL,"Project List");
  gtk_ctree_expand(projtree->projtree,projtree->projects);
}

void
gwf_projtree_construct (GwfProjTree *projtree,GwfProjlist *projlist,GwfTreeDataList *treedatas)
{
  projtree->projlist = projlist;
  projtree->treedatas = treedatas;
  gtk_signal_connect (GTK_OBJECT (projlist), "curproj_changed",
                      GTK_SIGNAL_FUNC (curproj_changed_cb),
                      (gpointer)projtree);
  gtk_signal_connect (GTK_OBJECT (projlist), "project_added",
                      GTK_SIGNAL_FUNC (project_added_cb),
                      (gpointer)projtree);
  gtk_signal_connect (GTK_OBJECT (projlist), "project_removed",
                      GTK_SIGNAL_FUNC (project_removed_cb),
                      (gpointer)projtree);
                      
  gwf_projlist_project_foreach(projtree->projlist,gwf_projtree_construct_tree,(gpointer)projtree);
}

GtkWidget*
gwf_projtree_new(GwfProjlist *projlist,GwfTreeDataList *treedatas)
{
  GwfProjTree	*projtree;
  
  projtree = GWF_PROJTREE( gtk_type_new (gwf_projtree_get_type()));
  gwf_projtree_construct(projtree,projlist,treedatas);
  return GTK_WIDGET ( projtree);
}

static void
gwf_projtree_destroy (GtkObject *object)
{
  GwfProjTree	*projtree = GWF_PROJTREE(object);
  
  gtk_signal_disconnect_by_data(GTK_OBJECT(projtree->projlist),(gpointer)projtree);
  gwf_projlist_project_foreach(projtree->projlist,gwf_projtree_destroy_tree,(gpointer)projtree);
  g_free(GWF_PROJTREE(object)->widname);
}

static void
gwf_projtree_construct_tree(GwfProject *project,gpointer user_data)
{
  GwfProjTree 	*projtree = user_data;
  GtkCTreeNode	*node;
  gchar		*name;
  
  name = gwf_project_get_widname(project);  
  node = gtk_ctree_add_node(projtree->projtree,projtree->projects,name);
  g_return_if_fail(node != NULL);
  gtk_ctree_node_set_row_data(projtree->projtree,node,(gpointer)project);

  gtk_signal_connect(GTK_OBJECT(project),"widname_changed",
                     GTK_SIGNAL_FUNC(widname_changed_cb),
                     user_data);
  gwf_projtree_add_treedata(projtree,project);
}

static void
gwf_projtree_destroy_tree (GwfProject *project,gpointer user_data)
{
  GwfProjTree	*projtree = user_data;
  gtk_signal_disconnect_by_data(GTK_OBJECT(project),user_data);
  gwf_treedata_list_foreach(projtree->treedatas,gwf_projtree_destroy_foreach_treedata,NULL);
}

/************************/
/* Treedata interaction */
/************************/
static void
gwf_projtree_add_treedata(GwfProjTree *projtree,GwfProject *project)
{
  GwfProjTreeData	data;

  data.tree = projtree->projtree;
  data.project = project;

  gwf_treedata_list_foreach(projtree->treedatas,gwf_projtree_create_foreach_treedata,(gpointer)&data);
}

static void
gwf_projtree_create_foreach_treedata(GwfTreeData *data,gpointer user_data)
{
  gwf_treedata_attach(data,((GwfProjTreeData*)user_data)->project,((GwfProjTreeData*)user_data)->tree);
}

static void
gwf_projtree_destroy_foreach_treedata(GwfTreeData *data,gpointer user_data)
{
  gwf_treedata_unregister_all_ptdata(data);
}

/************************************/
/* Project & Project list callbacks */
/************************************/

static void
curproj_changed_cb	(GwfProjlist *projlist, gchar *name,gpointer user_data)
{
  GwfProjTree 	*projtree = user_data;
  GwfProject	*project;
  GtkCTreeNode	*node;
  
  if (name)
  {
    project = gwf_projlist_get_project(projlist,name);
    node = gtk_ctree_find_project(projtree->projtree,project);
    g_return_if_fail(node != NULL);
    gtk_ctree_expand(projtree->projtree,node);
  }
}

static void
project_added_cb	(GwfProjlist *projlist, gchar *name,gpointer user_data)
{
  GwfProject	*project;
  
  project = gwf_projlist_get_project(projlist,name);
  g_return_if_fail(project != NULL);
  gwf_projtree_construct_tree(project,user_data);
}

static void
project_removed_cb	(GwfProjlist *projlist, gchar *name,gpointer user_data)
{
  GwfProjTree 	*projtree = user_data;
  GwfProject	*project;

  project = gwf_projlist_get_project(projlist,name);
  g_return_if_fail(project != NULL);
  gtk_ctree_remove_node (projtree->projtree,gtk_ctree_find_project(projtree->projtree,project));
}

static void
widname_changed_cb	(GwfProject *project, gchar *name,gpointer user_data)
{
  GwfProjTree 	*projtree = user_data;
  GtkCTreeNode	*node;

  node = gtk_ctree_find_project(projtree->projtree,project);
  g_return_if_fail(node != NULL);
  gtk_ctree_alter_node(projtree->projtree,node,name);
}

/****************/
/* GUI Creation */
/****************/
static void
gwf_projtree_interface (GwfProjTree *projtree)
{
  GtkWidget	*label;
  GtkWidget	*ctree;
  GtkAdjustment	*hadj,*vadj;

  hadj = GTK_ADJUSTMENT(gtk_adjustment_new(0,0,0,0,0,0));
  vadj = GTK_ADJUSTMENT(gtk_adjustment_new(0,0,0,0,0,0));

  gtk_scrolled_window_set_hadjustment(GTK_SCROLLED_WINDOW(projtree),hadj);
  gtk_scrolled_window_set_vadjustment(GTK_SCROLLED_WINDOW(projtree),vadj);
  gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(projtree),GTK_POLICY_AUTOMATIC,GTK_POLICY_AUTOMATIC);

  ctree = gtk_ctree_new (1, 0);
  projtree->projtree=GTK_CTREE(ctree);
  gtk_widget_show (ctree);
  gtk_container_add (GTK_CONTAINER (projtree), ctree);
  gtk_clist_set_column_width (GTK_CLIST (ctree), 0, 80);
  gtk_clist_column_titles_show (GTK_CLIST (ctree));
  gtk_clist_set_column_auto_resize (GTK_CLIST(ctree),0,TRUE);

  label = gtk_label_new ("Project information :");
  gtk_widget_show (label);
  gtk_clist_set_column_widget (GTK_CLIST (ctree), 0, label);

  gtk_widget_set_usize(ctree,150,-1);

  gtk_signal_connect(GTK_OBJECT(ctree), "tree-select-row",
  		     GTK_SIGNAL_FUNC(tree_row_selected),
  		     (gpointer)projtree);
}

static void
tree_row_selected(GtkCTree *ctree,GList *node,gint column,gpointer user_data)
{
  GtkCTreeRow		*row;
  GtkCTreeNode		*nod;
  GwfProject		*project;
  GwfProjTree		*projtree = user_data;
  gchar			*name = NULL;

  row = GTK_CTREE_ROW(node);

  if (!row->parent)  /* Root selected */
    return;

  nod = GTK_CTREE_NODE(node);
  while (!name)
  {
    do
    {
      project = gtk_ctree_node_get_row_data(ctree,nod);
      nod = row->parent;
      row = GTK_CTREE_ROW(nod);
    } while (row && row->parent && !project);

    if (project && GWF_IS_PROJECT(project))
    {
      name = gwf_project_get_widname(project);
    }
    else
    {
      if (!row || !row->parent)
        return;
    }

  }

  gwf_projlist_set_current(projtree->projlist,name);
}


