/*  Gtk+ Widget Factory
 *  Copyright (C) 1999-2000  Jeroen Benckhuijsen
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include "gwf_projlist.h"
#include "gwf_project.h"
#include <gtk/gtk.h>

enum {
  PROJECT_ADD,
  PROJECT_REMOVE,
  CURPROJ_CHANGED,
  LAST_SIGNAL
};

typedef struct _GwfProjlistForeach	GwfProjlistForeach;

struct _GwfProjlistForeach
{
  GwfProjlistForeachFunc	func;
  gpointer user_data;
};


static void		gwf_projlist_class_init		(GwfProjlistClass	*klass);
static void		gwf_projlist_init		(GwfProjlist		*projlist);
static void		gwf_projlist_destroy		(GtkObject		*object);
static gint		cmp_project_widname 		(GList 			*node,
   							 gchar 			*name);
static void		gwf_projlist_destroy_one 	(gpointer 		data,
   							 gpointer 		user_data);
static void		gwf_projlist_cb_project_change 	(GwfProject 		*project,
                                                         gpointer 		user_data);
static void		gwf_projlist_cb_project_destroy (GtkObject 		*object, 
   							 gpointer 		user_data);
static void		foreach_project 		(gpointer data,gpointer user_data);

static GtkObjectClass *parent_class = NULL;

static guint projlist_signals[LAST_SIGNAL] = { 0 };

guint 
gwf_projlist_get_type (void)
{
  static guint gwf_projlist_type = 0;

  if (!gwf_projlist_type)
    {
      GtkTypeInfo gwf_projlist_info =
      {
	"GwfProjlist",
	sizeof (GwfProjlist),
	sizeof (GwfProjlistClass),
	(GtkClassInitFunc) gwf_projlist_class_init,
	(GtkObjectInitFunc) gwf_projlist_init,
	/* reserved_1 */ NULL,
	/* reserved_2 */ NULL,
	(GtkClassInitFunc) NULL,
      };

      gwf_projlist_type = gtk_type_unique (gtk_object_get_type (),
					   &gwf_projlist_info);
    }

  return gwf_projlist_type;
}

GwfProjlist* 
gwf_projlist_new (void)
{
  return GWF_PROJLIST( gtk_type_new (gwf_projlist_get_type()));
}

static void
gwf_projlist_class_init	(GwfProjlistClass *klass)
{
  GtkObjectClass *object_class;

  object_class = (GtkObjectClass *) klass;

  parent_class = gtk_type_class (gtk_object_get_type ());
  
  object_class->destroy = gwf_projlist_destroy;
  
  projlist_signals [PROJECT_ADD] =
    gtk_signal_new ("project_added",
                    GTK_RUN_FIRST,
                    object_class->type,
                    GTK_SIGNAL_OFFSET (GwfProjlistClass, project_added),
                    gtk_marshal_NONE__STRING,
                    GTK_TYPE_NONE,1,
                    GTK_TYPE_STRING);
  projlist_signals [PROJECT_REMOVE] =
    gtk_signal_new ("project_removed",
                    GTK_RUN_FIRST,
                    object_class->type,
                    GTK_SIGNAL_OFFSET (GwfProjlistClass, project_removed),
                    gtk_marshal_NONE__STRING,
                    GTK_TYPE_NONE,1,
                    GTK_TYPE_STRING);
  projlist_signals [CURPROJ_CHANGED] =
    gtk_signal_new ("curproj_changed",
                    GTK_RUN_FIRST,
                    object_class->type,
                    GTK_SIGNAL_OFFSET (GwfProjlistClass, curproj_changed),
                    gtk_marshal_NONE__STRING,
                    GTK_TYPE_NONE,1,
                    GTK_TYPE_STRING);

  gtk_object_class_add_signals (object_class, projlist_signals, LAST_SIGNAL);

  klass->project_added = NULL;
  klass->project_removed = NULL;
}

static void
gwf_projlist_init (GwfProjlist *projlist)
{
  projlist->projects = NULL;
  projlist->changed = 0;
  projlist->idcount = 0;
  projlist->current = NULL;
}

static void
gwf_projlist_destroy (GtkObject *object)
{
  GList		*temp;
  
  temp = g_list_copy(GWF_PROJLIST(object)->projects);
  g_list_foreach(temp,gwf_projlist_destroy_one,NULL);
  g_list_free(GWF_PROJLIST(object)->projects);
  g_list_free(temp);
  
  parent_class->destroy(object);
}

gint
gwf_projlist_new_unique_project	(GwfProjlist *projlist)
{
  gchar		*name;
  
  name = gwf_projlist_unique_widname(projlist);
  if (!name)
    return 0;
  
  gwf_projlist_new_project(projlist,name);
  g_free(name);
  
  return 1;
}

gint
gwf_projlist_new_project (GwfProjlist *projlist, gchar *name)
{
  GwfProject	*project;
  
  g_return_val_if_fail(projlist != NULL,0);
  g_return_val_if_fail(name != NULL,0);
  
  project = gwf_project_new();
  if (!project)
    return 0;
    
  gwf_project_set_widname(project,name);
  
  if (!gwf_projlist_add_project(projlist,project))
  {
    gtk_object_destroy(GTK_OBJECT(project));
    return 0;
  }
  
  return 1;
}

gint
gwf_projlist_add_project (GwfProjlist *projlist, GwfProject *project)
{
  if (gwf_projlist_get_project(projlist,gwf_project_get_widname(project)))
    return 0;
  
  projlist->projects = g_list_append(projlist->projects,(gpointer)project);
  
  gtk_signal_connect(GTK_OBJECT(project),"project_changed",
                     GTK_SIGNAL_FUNC(gwf_projlist_cb_project_change),
                     (gpointer)projlist);
  gtk_signal_connect(GTK_OBJECT(project),"destroy",
                     GTK_SIGNAL_FUNC(gwf_projlist_cb_project_destroy),
                     (gpointer)projlist);
  
  projlist->changed++;
  
  gtk_signal_emit(GTK_OBJECT(projlist),projlist_signals[PROJECT_ADD],gwf_project_get_widname(project));
  
  gwf_projlist_set_current(projlist,gwf_project_get_widname(project));
  
  return 1;
}

void
gwf_projlist_remove_project (GwfProjlist *projlist, gchar *name)
{
  GwfProject	*tmp;

  g_return_if_fail(projlist != NULL);
  g_return_if_fail(name != NULL);
  
  tmp = gwf_projlist_get_project(projlist,name);
  if (!tmp)
    return;
  
  gtk_signal_emit(GTK_OBJECT(projlist),projlist_signals[PROJECT_REMOVE],name);
  gwf_projlist_set_current(projlist,NULL);
  
  if (gwf_project_get_changed(tmp))
    projlist->changed--;
      
  projlist->projects = g_list_remove(projlist->projects,(gpointer)tmp);
  gtk_object_destroy(GTK_OBJECT(tmp));
}

GwfProject*
gwf_projlist_get_project (GwfProjlist *projlist, gchar *name)
{
  GList 	*tmp_list;

  g_return_val_if_fail(projlist != NULL,NULL);
  g_return_val_if_fail(name != NULL,NULL);
  
  for (tmp_list = projlist->projects ; tmp_list ; tmp_list = g_list_next(tmp_list))
    if (cmp_project_widname(tmp_list,name))
      return GWF_PROJECT(tmp_list->data);
    
  return NULL;
}

gint
gwf_projlist_project_changed (GwfProjlist *projlist)
{
  g_return_val_if_fail(projlist != NULL,0);
  
  return projlist->changed;
}

void 
gwf_projlist_project_foreach	(GwfProjlist *projlist,GwfProjlistForeachFunc func,gpointer user_data)
{
  GwfProjlistForeach	tmp;
  
  g_return_if_fail(projlist != NULL);
  
  tmp.func = func;
  tmp.user_data = user_data;
  
  g_list_foreach(projlist->projects,foreach_project,(gpointer)&tmp);
}

gchar*		
gwf_projlist_unique_widname (GwfProjlist *projlist)
{
  gchar		*tmp;
  
  for ( ; projlist->idcount != G_MAXINT ; )
  {
    tmp = g_strdup_printf("GtkWidget%d",projlist->idcount++);
    if (!gwf_projlist_get_project(projlist,tmp))
      return tmp;
    g_free(tmp);
  }
  return NULL;
}

gint
gwf_projlist_project_change_name(GwfProjlist *projlist, gchar *oldname, gchar *newname)
{
  GwfProject		*project;
  
  g_return_val_if_fail(projlist != NULL,0);
  g_return_val_if_fail(oldname != NULL,0);
  g_return_val_if_fail(newname != NULL,0);
  
  project = gwf_projlist_get_project(projlist,oldname);
  if (!project)
    return 0;
    
  if (gwf_projlist_get_project(projlist,newname))
    return 0;
    
  gwf_project_set_widname(project,newname);
  return 1;
}

void
gwf_projlist_set_current (GwfProjlist *projlist, gchar *name)
{
  g_return_if_fail(projlist != NULL);
  
  if (name)
  {
    projlist->current = gwf_projlist_get_project(projlist,name);
    gtk_signal_emit(GTK_OBJECT(projlist),projlist_signals[CURPROJ_CHANGED],name);
  }
  else
  {
    projlist->current = NULL;
    gtk_signal_emit(GTK_OBJECT(projlist),projlist_signals[CURPROJ_CHANGED],NULL);
  }
}

GwfProject*
gwf_projlist_get_current (GwfProjlist *projlist)
{
  g_return_val_if_fail(projlist != NULL,NULL);
  
  return projlist->current;
}

gint
gwf_projlist_project_is_current	(GwfProjlist *projlist,GwfProject *project)
{
  g_return_val_if_fail(projlist != NULL,0);
  return (projlist->current == project);
}

static gint
cmp_project_widname (GList *node,gchar *name)
{
  GwfProject	*tmp = GWF_PROJECT(node->data);
 
  if (strcmp(gwf_project_get_widname(tmp),name) == 0) return 1;
  return 0;
}

static void
gwf_projlist_destroy_one (gpointer data,gpointer user_data)
{
  gtk_object_destroy(GTK_OBJECT(data));
}

static void
gwf_projlist_cb_project_change (GwfProject *project,gpointer user_data)
{
  if (gwf_project_get_changed(project))
    GWF_PROJLIST(user_data)->changed++;
  else
    GWF_PROJLIST(user_data)->changed--;
}

static void
gwf_projlist_cb_project_destroy (GtkObject *object, gpointer user_data)
{
  gwf_projlist_remove_project (GWF_PROJLIST(user_data),gwf_project_get_widname(GWF_PROJECT(object)));
}

static void
foreach_project (gpointer data,gpointer user_data)
{
  GwfProjlistForeach	*tmp = user_data;
  
  tmp->func((GwfProject*)data,tmp->user_data);
}


